package ComplexMatricesTest;

import Matrices.ComplexMatrices.*;
import Matrices.Matrices.*;
import cComplexNumbers.ComplexNumbers.*;
import pPolynomials.ComplexPolynomials.ComplexPolynomials;
import pPolynomials.ComplexPolynomials.pComplexPolynomial;
import references.references.*;

import static Matrices.ComplexMatrices.ComplexMatrices.*;
import static Matrices.Matrices.Matrices.CreateMatrix;
import static Matrices.Matrices.Matrices.CreateSquareMatrix;
import static Matrices.Matrices.Matrices.MatrixEqualsEpsilon;
import static MatricesTest.MatricesTest.*;
import static cComplexNumbers.ComplexNumbers.ComplexNumbers.cAssignComplex;
import static cComplexNumbers.ComplexNumbers.ComplexNumbers.cCreateComplexNumber;
import static cComplexNumbers.ComplexNumbers.ComplexNumbers.cEpsilonCompareComplex;
import static mathmath.math.math.*;
import static nnumbers.StringToNumber.StringToNumber.nStringToNumberArray;
import static pPolynomials.ComplexPolynomials.ComplexPolynomials.pCreateComplexPolynomial;
import static testing.testing.testing.*;

public class ComplexMatricesTest {
    public static void testComplexMatrices(NumberReference failures) {
        testComplexDeterminant(failures);
        testComplexInverse(failures);
        //testComplexQRDecomposition(failures);
        //testComplexQRDecomposition2(failures);
    }

    public static void testComplexDeterminant(NumberReference failures) {
        Matrix a;
        ComplexMatrix ca;
        cComplexNumber d;

        a = createExample4x4_1();
        ca = CreateComplexMatrixFromMatrix(a);

        d = DeterminantComplex(ca);

        AssertTrue(mathEpsilonCompare(d.re, 88d, 0.00000000001), failures);
        AssertTrue(mathEpsilonCompare(d.im, 0d, 0.00000000001), failures);

        ca = CreateExampleComplexMatrix2x2();

        d = DeterminantComplex(ca);

        AssertTrue(mathEpsilonCompare(d.re, 0d, 0.00000000001), failures);
        AssertTrue(mathEpsilonCompare(d.im, -1d, 0.00000000001), failures);
    }

    public static ComplexMatrix CreateExampleComplexMatrix2x2() {
        ComplexMatrix ca;
        ca = CreateComplexMatrix(2d, 2d);
        IndexComplex(ca, 0d, 0d).re = 0d;
        IndexComplex(ca, 0d, 0d).im = 1d;
        IndexComplex(ca, 0d, 1d).re = 1d;
        IndexComplex(ca, 0d, 1d).im = 0d;
        IndexComplex(ca, 1d, 0d).re = 0d;
        IndexComplex(ca, 1d, 0d).im = 0d;
        IndexComplex(ca, 1d, 1d).re = -1d;
        IndexComplex(ca, 1d, 1d).im = 0d;
        return ca;
    }

    public static ComplexMatrix CreateExampleComplexMatrix2x2Inverse() {
        ComplexMatrix ca;
        ca = CreateComplexMatrix(2d, 2d);
        IndexComplex(ca, 0d, 0d).re = 0d;
        IndexComplex(ca, 0d, 0d).im = -1d;
        IndexComplex(ca, 0d, 1d).re = 0d;
        IndexComplex(ca, 0d, 1d).im = -1d;
        IndexComplex(ca, 1d, 0d).re = 0d;
        IndexComplex(ca, 1d, 0d).im = 0d;
        IndexComplex(ca, 1d, 1d).re = -1d;
        IndexComplex(ca, 1d, 1d).im = 0d;
        return ca;
    }

    public static void testComplexInverse(NumberReference failures) {
        Matrix a, f;
        ComplexMatrix ca, b, cf;

        a = createExample4x4_1();
        ca = CreateComplexMatrixFromMatrix(a);

        b = CreateSquareComplexMatrix(4d);
        InverseComplex(ca, b);

        f = createAnswerInverse4x4_1();
        cf = CreateComplexMatrixFromMatrix(f);

        AssertTrue(ComplexMatrixEqualsEpsilon(b, cf, 0.00000000001), failures);

        ca = CreateExampleComplexMatrix2x2();

        b = CreateSquareComplexMatrix(2d);
        InverseComplex(ca, b);

        cf = CreateExampleComplexMatrix2x2Inverse();

        AssertTrue(ComplexMatrixEqualsEpsilon(b, cf, 0.00000000001), failures);
    }

    /*public static void testComplexQRDecomposition(NumberReference failures) {
        ComplexMatrix ca, cq, cr;
        Matrix a, fq, fr, r, q;

        // Create example:
        a = CreateSquareMatrix(3d);

        a.r[0].c = nStringToNumberArray("12, -51,   4".toCharArray());
        a.r[1].c = nStringToNumberArray(" 6, 167, -68".toCharArray());
        a.r[2].c = nStringToNumberArray("-4,  24, -41".toCharArray());

        ca = CreateComplexMatrixFromMatrix(a);
        cq = CreateSquareComplexMatrix(3d);
        cr = CreateSquareComplexMatrix(3d);

        QRDecompositionComplex(ca, cq, cr);

        fq = CreateSquareMatrix(3d);

        fq.r[0].c[0] = -6d/7d;
        fq.r[0].c[1] = 69d/175d;
        fq.r[0].c[2] = -58d/175;

        fq.r[1].c[0] = -3d/7d;
        fq.r[1].c[1] = -158d/175d;
        fq.r[1].c[2] = 6d/175d;

        fq.r[2].c[0] = 2d/7d;
        fq.r[2].c[1] = -6d/35d;
        fq.r[2].c[2] = -33d/35d;

        fr = CreateSquareMatrix(3d);

        fr.r[0].c = nStringToNumberArray("-14,  -21, 14".toCharArray());
        fr.r[1].c = nStringToNumberArray(" 0,  -175, 70".toCharArray());
        fr.r[2].c = nStringToNumberArray(" 0,     0,   35".toCharArray());

        r = CreateReMatrixFromComplexMatrix(cr);
        q = CreateReMatrixFromComplexMatrix(cq);

        AssertTrue(MatrixEqualsEpsilon(r, fr, 0.00000000001), failures);
        AssertTrue(MatrixEqualsEpsilon(q, fq, 0.00000000001), failures);
    }

    public static void testComplexQRDecomposition2(NumberReference failures) {
        ComplexMatrix ca, cq, cr, fr, fq;

        // Create example:
        ca = CreateSquareComplexMatrix(2d);

        ca.r[0].c[0] = cCreateComplexNumber(1d, 0d);
        ca.r[0].c[1] = cCreateComplexNumber(0d, 1d);

        ca.r[1].c[0] = cCreateComplexNumber(3d, 0d);
        ca.r[1].c[1] = cCreateComplexNumber(4d, 0d);

        cq = CreateSquareComplexMatrix(2d);
        cr = CreateSquareComplexMatrix(2d);

        QRDecompositionComplex(ca, cq, cr);

        fr = CreateSquareComplexMatrix(2d);

        fr.r[0].c[0] = cCreateComplexNumber(-3.16228d, 0d);
        fr.r[0].c[1] = cCreateComplexNumber(-3.79473d, -0.316228d);

        fr.r[1].c[0] = cCreateComplexNumber(0d, 0d);
        fr.r[1].c[1] = cCreateComplexNumber(-1.58114d, 0d);

        fq = CreateSquareComplexMatrix(2d);

        fq.r[0].c[0] = cCreateComplexNumber(-0.316228d, 0d);
        fq.r[0].c[1] = cCreateComplexNumber(0.758947d, 0.56921d);

        fq.r[1].c[0] = cCreateComplexNumber(-0.948683d, 0d);
        fq.r[1].c[1] = cCreateComplexNumber(-0.252982d, -0.189737d);

        AssertTrue(ComplexMatrixEqualsEpsilon(cr, fr, 0.0001), failures);
        AssertTrue(ComplexMatrixEqualsEpsilon(cq, fq, 0.0001), failures);
    }*/

    public static void testComplexCharacteristicPolynomial(NumberReference failures){
        ComplexMatrix a;
        pComplexPolynomial coeffs, answerCoeffs;
        double m;
        boolean success;

        a = CreateComplexMatrix(3d, 3d);
        IndexComplex(a, 0d, 0d).re = 0d;
        IndexComplex(a, 0d, 0d).im = 0d;
        IndexComplex(a, 0d, 1d).re = -4d;
        IndexComplex(a, 0d, 1d).im = 0d;
        IndexComplex(a, 0d, 2d).re = -6d;
        IndexComplex(a, 0d, 2d).im = 0d;
        IndexComplex(a, 1d, 0d).re = -1d;
        IndexComplex(a, 1d, 0d).im = 0d;
        IndexComplex(a, 1d, 1d).re = 0d;
        IndexComplex(a, 1d, 1d).im = 0d;
        IndexComplex(a, 1d, 2d).re = -3d;
        IndexComplex(a, 1d, 2d).im = 0d;
        IndexComplex(a, 2d, 0d).re = 1d;
        IndexComplex(a, 2d, 0d).im = 0d;
        IndexComplex(a, 2d, 1d).re = 2d;
        IndexComplex(a, 2d, 1d).im = 0d;
        IndexComplex(a, 2d, 2d).re = 5d;
        IndexComplex(a, 2d, 2d).im = 0d;

        coeffs = new pComplexPolynomial();
        ComplexCharacteristicPolynomial(a, coeffs);

        answerCoeffs = pCreateComplexPolynomial(3d);
        answerCoeffs.cs[0] = cCreateComplexNumber(-4d, 0d);
        answerCoeffs.cs[1] = cCreateComplexNumber(8d, 0d);
        answerCoeffs.cs[2] = cCreateComplexNumber(-5d, 0d);
        answerCoeffs.cs[3] = cCreateComplexNumber(1d, 0d);

        for(m = 0d; m < 4d; m = m + 1d){
            AssertTrue(cEpsilonCompareComplex(answerCoeffs.cs[(int)m], coeffs.cs[(int)m], 0.000001), failures);
        }
    }

    public static void testComplexCharacteristicPolynomial2(NumberReference failures){
        ComplexMatrix a;
        pComplexPolynomial p, ap;
        double m;

        a = CreateComplexMatrix(2d, 2d);
        IndexComplex(a, 0d, 0d).re = 0d;
        IndexComplex(a, 0d, 0d).im = 1d;
        IndexComplex(a, 0d, 1d).re = 0d;
        IndexComplex(a, 0d, 1d).im = 0d;
        IndexComplex(a, 1d, 0d).re = 0d;
        IndexComplex(a, 1d, 0d).im = 1d;
        IndexComplex(a, 1d, 1d).re = 1d;
        IndexComplex(a, 1d, 1d).im = 0d;

        p = new pComplexPolynomial();
        ComplexCharacteristicPolynomial(a, p);

        ap = pCreateComplexPolynomial(2d);
        ap.cs[0] = cCreateComplexNumber(0d, 1d);
        ap.cs[1] = cCreateComplexNumber(-1d, -1d);
        ap.cs[2] = cCreateComplexNumber(1d, 0d);

        for(m = 0d; m < 3d; m = m + 1d){
            AssertTrue(cEpsilonCompareComplex(ap.cs[(int)m], p.cs[(int)m], 0.000001), failures);
        }
    }

    public static void testComplexEigenValues(NumberReference failures){
        ComplexMatrix a;
        cComplexNumberArrayReference evsReference;
        cComplexNumber [] evs, answerEvs;
        double i, j;
        boolean found, success;


        a = CreateComplexMatrix(2d, 2d);
        IndexComplex(a, 0d, 0d).re = 0d;
        IndexComplex(a, 0d, 0d).im = 0d;
        IndexComplex(a, 0d, 1d).re = 1d;
        IndexComplex(a, 0d, 1d).im = 0d;
        IndexComplex(a, 1d, 0d).re = -2d;
        IndexComplex(a, 1d, 0d).im = 0d;
        IndexComplex(a, 1d, 1d).re = -3d;
        IndexComplex(a, 1d, 1d).im = 0d;

        evsReference = new cComplexNumberArrayReference();
        success = EigenvaluesComplex(a, evsReference);
        AssertTrue(success, failures);

        if(success) {
            evs = evsReference.complexNumbers;

            answerEvs = new cComplexNumber[(int) NumberOfRowsComplex(a)];
            answerEvs[0] = cCreateComplexNumber(-1d, 0d);
            answerEvs[1] = cCreateComplexNumber(-2d, 0d);

            for (i = 0d; i < 2d; i = i + 1d) {
                found = false;
                for (j = 0d; j < 2d; j = j + 1d) {
                    if (cEpsilonCompareComplex(evs[(int) i], answerEvs[(int) j], 0.00001)) {
                        found = true;
                    }
                }
                AssertTrue(found, failures);
            }
        }
    }

    public static void testComplexEigenValues2(NumberReference failures){
        ComplexMatrix a;
        cComplexNumberArrayReference evsReference;
        cComplexNumber [] evs, answerEvs;
        double i, j;
        boolean found, success;

        a = CreateComplex3x3Matrix();

        evsReference = new cComplexNumberArrayReference();
        success = EigenvaluesComplex(a, evsReference);
        AssertTrue(success, failures);

        if(success) {
            evs = evsReference.complexNumbers;

            answerEvs = new cComplexNumber[(int) NumberOfRowsComplex(a)];
            answerEvs[0] = cCreateComplexNumber(1d, 0d);
            answerEvs[1] = cCreateComplexNumber(-0.5d, 0.8660254037844386467637231d);
            answerEvs[2] = cCreateComplexNumber(-0.5d, -0.8660254037844386467637231d);

            for (i = 0d; i < 3d; i = i + 1d) {
                found = false;
                for (j = 0d; j < 3d; j = j + 1d) {
                    if (cEpsilonCompareComplex(evs[(int) i], answerEvs[(int) j], 0.00001)) {
                        found = true;
                    }
                }
                AssertTrue(found, failures);
            }
        }
    }

    public static ComplexMatrix CreateComplex3x3Matrix() {
        ComplexMatrix a;
        a = CreateComplexMatrix(3d, 3d);
        IndexComplex(a, 0d, 0d).re = 0d;
        IndexComplex(a, 0d, 0d).im = 0d;
        IndexComplex(a, 0d, 1d).re = 1d;
        IndexComplex(a, 0d, 1d).im = 0d;
        IndexComplex(a, 0d, 2d).re = 0d;
        IndexComplex(a, 0d, 2d).im = 0d;
        IndexComplex(a, 1d, 0d).re = 0d;
        IndexComplex(a, 1d, 0d).im = 0d;
        IndexComplex(a, 1d, 1d).re = 0d;
        IndexComplex(a, 1d, 1d).im = 0d;
        IndexComplex(a, 1d, 2d).re = 1d;
        IndexComplex(a, 1d, 2d).im = 0d;
        IndexComplex(a, 2d, 0d).re = 1d;
        IndexComplex(a, 2d, 0d).im = 0d;
        IndexComplex(a, 2d, 1d).re = 0d;
        IndexComplex(a, 2d, 1d).im = 0d;
        IndexComplex(a, 2d, 2d).re = 0d;
        IndexComplex(a, 2d, 2d).im = 0d;
        return a;
    }

    public static void testComplexInversePowerMethod(NumberReference failures){
        ComplexMatrix a, evecMatrix;
        cComplexNumberArrayReference evsReference;
        cComplexNumber eval, evalApprox, answerEvs;
        double i, N;
        boolean found, success, equals;
        cComplexNumberArrayReference eigenvector;

        a = CreateComplex3x3Matrix();

        eval = cCreateComplexNumber(-0.5d, 0.8660254037844386467637231d);
        evalApprox = cCreateComplexNumber(-0.5d * 1.1, 0.8660254037844386467637231d * 1.1);

        eigenvector = new cComplexNumberArrayReference();
        success = InversePowerMethodComplex(a, evalApprox, 100d, eigenvector);

        AssertTrue(success, failures);

        N = NumberOfRowsComplex(a);
        evecMatrix = CreateComplexMatrix(N, 1d);
        for(i = 0d; i < N; i = i + 1d){
            cAssignComplex(evecMatrix.r[(int)i].c[0], eigenvector.complexNumbers[(int)i]);
        }

        equals = CheckComplexEigenpairPrecision(a, eval, evecMatrix, 0.00001);

        AssertTrue(equals, failures);
    }

    public static void testComplexEigenpairs(NumberReference failures){
        ComplexMatrix a;
        double i;
        cComplexNumber eval;
        cComplexNumberArrayReference eigenvaluesReference;
        ComplexMatrixArrayReference eigenvectorsReference, eigenvectorsReference2;
        ComplexMatrix evec;
        boolean success, equal;

        a = CreateSquareComplexMatrix(3d);

        IndexComplex(a, 0d, 0d).re = 1d;
        IndexComplex(a, 0d, 0d).im = 0d;
        IndexComplex(a, 0d, 1d).re = 2d;
        IndexComplex(a, 0d, 1d).im = 0d;
        IndexComplex(a, 0d, 2d).re = 1d;
        IndexComplex(a, 0d, 2d).im = 0d;
        IndexComplex(a, 1d, 0d).re = 6d;
        IndexComplex(a, 1d, 0d).im = 0d;
        IndexComplex(a, 1d, 1d).re = -1d;
        IndexComplex(a, 1d, 1d).im = 0d;
        IndexComplex(a, 1d, 2d).re = 0d;
        IndexComplex(a, 1d, 2d).im = 0d;
        IndexComplex(a, 2d, 0d).re = -1d;
        IndexComplex(a, 2d, 0d).im = 0d;
        IndexComplex(a, 2d, 1d).re = -2d;
        IndexComplex(a, 2d, 1d).im = 0d;
        IndexComplex(a, 2d, 2d).re = -1d;
        IndexComplex(a, 2d, 2d).im = 0d;

        eigenvectorsReference = new ComplexMatrixArrayReference();
        eigenvaluesReference = new cComplexNumberArrayReference();
        eigenvectorsReference2 = new ComplexMatrixArrayReference();
        success = EigenpairsComplex(a, eigenvaluesReference, eigenvectorsReference);

        AssertTrue(success, failures);

        if(success) {
            // Check
            for(i = 0d; i < 3d; i = i + 1d){
                evec = eigenvectorsReference.matrices[(int)i];
                eval = eigenvaluesReference.complexNumbers[(int)i];

                equal = CheckComplexEigenpairPrecision(a, eval, evec, 0.001);

                AssertTrue(equal, failures);
            }
        }

        success = EigenvectorsComplex(a, eigenvectorsReference2);

        AssertTrue(success, failures);

        AssertTrue(ComplexMatrixEqualsEpsilon(eigenvectorsReference.matrices[0], eigenvectorsReference2.matrices[0], 0.001), failures);
        AssertTrue(ComplexMatrixEqualsEpsilon(eigenvectorsReference.matrices[1], eigenvectorsReference2.matrices[1], 0.001), failures);
        AssertTrue(ComplexMatrixEqualsEpsilon(eigenvectorsReference.matrices[2], eigenvectorsReference2.matrices[2], 0.001), failures);
    }
}
