package MatricesTest;

import Matrices.Matrices.*;
import references.references.*;

import static Matrices.Matrices.Matrices.*;
import static java.lang.Math.*;
import static mathmath.math.math.*;
import static nnumbers.StringToNumber.StringToNumber.*;
import static references.references.references.*;
import static strings.strings.strings.*;
import static testing.testing.testing.*;

public class MatricesTest {
    public static void testDeterminant(NumberReference failures){
        Matrix a;
        double d;

        a = createExample4x4_1();

        d = Determinant(a);

        AssertTrue(mathEpsilonCompare(d, 88d, 0.00000000001), failures);
    }

    public static void testInverse(NumberReference failures){
        Matrix a, b, f;

        a = createExample4x4_1();

        b = CreateSquareMatrix(4d);
        Inverse(a, b);

        f = createAnswerInverse4x4_1();

        AssertTrue(MatrixEqualsEpsilon(b, f, 0.00000000001), failures);
    }

    public static Matrix createAnswerInverse4x4_1() {
        Matrix a;
        MatrixReference aref;
        char [] s;

        s = new char [0];
        s = AppendString(s, "-0.13636363636363635,  0.8636363636363636,   -0.6818181818181818,   -0.4090909090909091 \n".toCharArray());
        s = AppendString(s, "-0.6363636363636364,   2.3636363636363638,   -0.9318181818181818,   -0.6590909090909091 \n".toCharArray());
        s = AppendString(s, " 0.045454545454545456, 0.045454545454545456, -0.022727272727272728, -0.11363636363636363 \n".toCharArray());
        s = AppendString(s, " 0.045454545454545456, 0.045454545454545456,  0.22727272727272727,   0.13636363636363635 \n".toCharArray());

        aref = new MatrixReference();
        ParseMatrixFromString(aref, s, new StringReference());
        a = aref.matrix;

        delete(aref);

        return a;
    }

    public static Matrix createExample4x4_1() {
        Matrix a;
        MatrixReference aref;
        char [] s;

        s = new char [0];
        s = AppendString(s, " 5, -2,  2, 7 \n".toCharArray());
        s = AppendString(s, " 1,  0,  0, 3 \n".toCharArray());
        s = AppendString(s, "-3,  1,  5, 0 \n".toCharArray());
        s = AppendString(s, " 3, -1, -9, 4 \n".toCharArray());

        aref = new MatrixReference();
        ParseMatrixFromString(aref, s, new StringReference());
        a = aref.matrix;

        delete(aref);

        return a;
    }

    private static void delete(Object o) {
        // Java has garbage collection.
    }

    public static void testInverseAlt2(NumberReference failures){
        Matrix a, b, f;

        a = createExample4x4_1();

        b = InverseUsingCharacteristicPolynomial(a);

        f = createAnswerInverse4x4_1();

        AssertTrue(MatrixEqualsEpsilon(b, f, 0.00000000001), failures);
    }

    public static void TestCholesky(NumberReference failures){
        Matrix a, b, f;

        a = CreateSquareMatrix(3d);

        a.r[0].c = nStringToNumberArray("4, 1,  1".toCharArray());
        a.r[1].c = nStringToNumberArray("1, 5,  3".toCharArray());
        a.r[2].c = nStringToNumberArray("1, 3, 15".toCharArray());

        b = CreateSquareMatrix(3d);
        Cholesky(a, b);

        f = CreateSquareMatrix(3d);

        f.r[0].c = nStringToNumberArray("2,    0,       0".toCharArray());
        f.r[1].c = nStringToNumberArray("0.5,  2.17945, 0".toCharArray());
        f.r[2].c = nStringToNumberArray("0.5,  1.26179, 3.62738".toCharArray());

        AssertTrue(MatrixEqualsEpsilon(b, f, 0.001), failures);
    }

    public static void testCharacteristicPolynomial(NumberReference failures){
        Matrix a;
        double [] coeffs, answerCoeffs;
        double m;

        a = CreateSquareMatrix(3d);

        a.r[0].c = nStringToNumberArray(" 0, -4, -6".toCharArray());
        a.r[1].c = nStringToNumberArray("-1,  0, -3".toCharArray());
        a.r[2].c = nStringToNumberArray(" 1,  2,  5".toCharArray());

        coeffs = CharacteristicPolynomial(a);

        answerCoeffs = nStringToNumberArray("-4, 8, -5, 1".toCharArray());

        for(m = 0d; m < 4d; m = m + 1d){
            AssertTrue(mathEpsilonCompare(answerCoeffs[(int)m], coeffs[(int)m], 0.000001), failures);
        }
    }

    public static void testCharacteristicPolynomial2(NumberReference failures){
        Matrix a;
        double [] coeffs, answerCoeffs;
        double m;

        a = CreateSquareMatrix(3d);

        a.r[0].c = nStringToNumberArray(" 3,  1,  5".toCharArray());
        a.r[1].c = nStringToNumberArray(" 3,  3,  1".toCharArray());
        a.r[2].c = nStringToNumberArray(" 4,  6,  4".toCharArray());

        coeffs = CharacteristicPolynomial(a);

        answerCoeffs = new double [(int)(NumberOfRows(a) + 1d)];
        answerCoeffs[0] = -40;
        answerCoeffs[1] = 4;
        answerCoeffs[2] = -10;
        answerCoeffs[3] = 1;

        for(m = 0d; m < 4d; m = m + 1d){
            AssertTrue(mathEpsilonCompare(answerCoeffs[(int)m], coeffs[(int)m], 0.000001), failures);
        }
    }

    public static void testCharacteristicPolynomial3(NumberReference failures){
        Matrix a;
        double [] coeffs, answerCoeffs;
        double m;

        failures = CreateNumberReference(0d);

        a = CreateSquareMatrix(3d);

        a.r[0].c = nStringToNumberArray(" 1, -3,  3".toCharArray());
        a.r[1].c = nStringToNumberArray(" 3, -5,  3".toCharArray());
        a.r[2].c = nStringToNumberArray(" 6, -6,  4".toCharArray());

        coeffs = CharacteristicPolynomial(a);

        answerCoeffs = nStringToNumberArray("-16, -12, 0, 1".toCharArray());

        for(m = 0d; m < 4d; m = m + 1d){
            AssertTrue(mathEpsilonCompare(answerCoeffs[(int)m], coeffs[(int)m], 0.000001), failures);
        }
    }

    public static void testEigenValues(NumberReference failures){
        Matrix a;
        NumberArrayReference evsReference;
        double [] evs, answerEvs;
        double i, j, m;
        boolean found, success;

        a = CreateSquareMatrix(2d);

        a.r[0].c = nStringToNumberArray(" 0,  1".toCharArray());
        a.r[1].c = nStringToNumberArray("-2, -3".toCharArray());

        evsReference = new NumberArrayReference();
        success = Eigenvalues(a, evsReference);
        AssertTrue(success, failures);

        if(success) {
            evs = evsReference.numberArray;

            answerEvs = new double[(int) NumberOfRows(a)];
            answerEvs[0] = -1;
            answerEvs[1] = -2;

            for (i = 0d; i < 2d; i = i + 1d) {
                found = false;
                for (j = 0d; j < 2d; j = j + 1d) {
                    if (mathEpsilonCompare(evs[(int) i], answerEvs[(int) j], 0.00001)) {
                        found = true;
                    }
                }
                AssertTrue(found, failures);
            }
        }
    }

    public static void testEigenValues2(NumberReference failures){
        Matrix a;
        NumberArrayReference evsReference;
        double [] evs, answerEvs;
        boolean success;

        a = CreateSquareMatrix(3d);

        a.r[0].c = nStringToNumberArray(" 1, -3,  3".toCharArray());
        a.r[1].c = nStringToNumberArray(" 3, -5,  3".toCharArray());
        a.r[2].c = nStringToNumberArray(" 6, -6,  4".toCharArray());

        evsReference = new NumberArrayReference();
        success = Eigenvalues(a, evsReference);

        AssertTrue(success, failures);

        if(success) {
            evs = evsReference.numberArray;

            answerEvs = new double[(int) NumberOfRows(a)];
            answerEvs[0] = -2;
            answerEvs[1] = -2;
            answerEvs[2] = 4;

            SetsEqualEpsilon(evs, answerEvs, 0.001, failures);
        }
    }

    public static void testGaussianEliminationInto(NumberReference failures){
        Matrix a, f;

        a = CreateSquareMatrix(3d);

        a.r[0].c = nStringToNumberArray(" 1,  3,  1".toCharArray());
        a.r[1].c = nStringToNumberArray(" 1,  1, -1".toCharArray());
        a.r[2].c = nStringToNumberArray(" 3, 11,  5".toCharArray());

        GaussianElimination(a);

        f = CreateSquareMatrix(3d);

        f.r[0].c = nStringToNumberArray(" 3,  11,  5".toCharArray());
        f.r[1].c = nStringToNumberArray(" 0,  -2.6666666666666, -2.6666666666666".toCharArray());
        f.r[2].c = nStringToNumberArray(" 0, 0,  0".toCharArray());

        AssertTrue(MatrixEqualsEpsilon(a, f,  0.001), failures);
    }

    public static void testEigenpairs(NumberReference failures){
        Matrix a;
        double i, eval;
        NumberArrayReference eigenvaluesReference;
        MatrixArrayReference eigenvectorsReference, eigenvectorsReference2;
        Matrix evec;
        boolean success, equal;

        a = CreateSquareMatrix(3d);

        a.r[0].c = nStringToNumberArray(" 1,  2,  1".toCharArray());
        a.r[1].c = nStringToNumberArray(" 6, -1,  0".toCharArray());
        a.r[2].c = nStringToNumberArray("-1, -2, -1".toCharArray());

        eigenvectorsReference = new MatrixArrayReference();
        eigenvectorsReference2 = new MatrixArrayReference();
        eigenvaluesReference = new NumberArrayReference();
        success = Eigenpairs(a, eigenvaluesReference, eigenvectorsReference);

        AssertTrue(success, failures);

        if(success) {
            // Check
            for(i = 0d; i < 3d; i = i + 1d){
                evec = eigenvectorsReference.matrices[(int)i];
                eval = eigenvaluesReference.numberArray[(int)i];

                equal = CheckEigenpairPrecision(a, eval, evec, 0.001);

                AssertTrue(equal, failures);
            }
        }

        success = Eigenvectors(a, eigenvectorsReference2);

        //System.out.println(new String(MatrixArrayToString(eigenvectorsReference2.matrices, 3d)));

        AssertTrue(success, failures);

        AssertTrue(MatrixEqualsEpsilon(eigenvectorsReference.matrices[0], eigenvectorsReference2.matrices[0], 0.001), failures);
        AssertTrue(MatrixEqualsEpsilon(eigenvectorsReference.matrices[1], eigenvectorsReference2.matrices[1], 0.001), failures);
        AssertTrue(MatrixEqualsEpsilon(eigenvectorsReference.matrices[2], eigenvectorsReference2.matrices[2], 0.001), failures);
    }

    public static void testQRDecomposition(NumberReference failures) {
        Matrix a, fq, fr, r, q;

        r = CreateSquareMatrix(3d);
        q = CreateSquareMatrix(3d);

        // Create example:
        a = CreateSquareMatrix(3d);

        a.r[0].c = nStringToNumberArray("12, -51,   4".toCharArray());
        a.r[1].c = nStringToNumberArray(" 6, 167, -68".toCharArray());
        a.r[2].c = nStringToNumberArray("-4,  24, -41".toCharArray());

        QRDecomposition(a, q, r);

        fq = CreateSquareMatrix(3d);

        fq.r[0].c[0] = 6d/7d;
        fq.r[0].c[1] = -69d/175d;
        fq.r[0].c[2] = -58d/175d;

        fq.r[1].c[0] = 3d/7d;
        fq.r[1].c[1] = 158d/175d;
        fq.r[1].c[2] = 6d/175d;

        fq.r[2].c[0] = -2d/7d;
        fq.r[2].c[1] = 6d/35d;
        fq.r[2].c[2] = -33d/35d;

        fr = CreateSquareMatrix(3d);

        fr.r[0].c = nStringToNumberArray("14,  21, -14".toCharArray());
        fr.r[1].c = nStringToNumberArray(" 0, 175, -70".toCharArray());
        fr.r[2].c = nStringToNumberArray(" 0,   0,  35".toCharArray());

        AssertTrue(MatrixEqualsEpsilon(r, fr, 0.000000000001), failures);
        AssertTrue(MatrixEqualsEpsilon(q, fq, 0.000000000001), failures);
    }

    public static void testQRDecomposition2(NumberReference failures) {
        Matrix a, fq, fr, r, q;

        r = CreateSquareMatrix(2d);
        q = CreateSquareMatrix(2d);

        // Create example:
        a = CreateSquareMatrix(2d);

        a.r[0].c = nStringToNumberArray(" 0,  1".toCharArray());
        a.r[1].c = nStringToNumberArray("-2, -3".toCharArray());

        QRDecomposition(a, q, r);

        fq = CreateSquareMatrix(2d);

        fq.r[0].c[0] = 0d;
        fq.r[0].c[1] = 1d;

        fq.r[1].c[0] = -1d;
        fq.r[1].c[1] = 0d;

        fr = CreateSquareMatrix(2d);

        fr.r[0].c = nStringToNumberArray("2, 3".toCharArray());
        fr.r[1].c = nStringToNumberArray("0, 1".toCharArray());

        AssertTrue(MatrixEqualsEpsilon(r, fr, 0.0000000000001), failures);
        AssertTrue(MatrixEqualsEpsilon(q, fq, 0.0000000000001), failures);
    }

    public static void testQRDecomposition3(NumberReference failures) {
        Matrix a, fq, fr, r, q;

        r = CreateSquareMatrix(3d);
        q = CreateMatrix(4d, 3d);

        // Create example:
        a = CreateMatrix(4d, 3d);

        a.r[0].c = nStringToNumberArray(" -1, -1, 1".toCharArray());
        a.r[1].c = nStringToNumberArray("  1,  3, 3".toCharArray());
        a.r[2].c = nStringToNumberArray(" -1, -1, 5".toCharArray());
        a.r[3].c = nStringToNumberArray("  1,  3, 7".toCharArray());

        QRDecomposition(a, q, r);

        fq = CreateMatrix(4d, 3d);

        fq.r[0].c = nStringToNumberArray("-0.5, 0.5, -0.5".toCharArray());
        fq.r[1].c = nStringToNumberArray(" 0.5, 0.5, -0.5".toCharArray());
        fq.r[2].c = nStringToNumberArray("-0.5, 0.5,  0.5".toCharArray());
        fq.r[3].c = nStringToNumberArray(" 0.5, 0.5,  0.5".toCharArray());

        fr = CreateSquareMatrix(3d);

        fr.r[0].c = nStringToNumberArray(" 2, 4, 2".toCharArray());
        fr.r[1].c = nStringToNumberArray(" 0, 2, 8".toCharArray());
        fr.r[2].c = nStringToNumberArray(" 0, 0, 4".toCharArray());

        AssertTrue(MatrixEqualsEpsilon(r, fr, 0.0000000000001), failures);
        AssertTrue(MatrixEqualsEpsilon(q, fq, 0.0000000000001), failures);
    }

    public static void testEigenValues3QR(NumberReference failures) {
        Matrix a, x, q, r;
        double [] evs, answerEvs;
        boolean  success;

        a = CreateSquareMatrix(2d);

        a.r[0].c = nStringToNumberArray(" 0,  1".toCharArray());
        a.r[1].c = nStringToNumberArray("-2, -3".toCharArray());

        x = CreateMatrix(2d, 2d);
        q = CreateMatrix(2d, 2d);
        r = CreateMatrix(2d, 2d);
        success = QRAlgorithm(a, r, x, q, 0.00001, 100d);
        AssertTrue(success, failures);

        if(success) {
            evs = new double [2];
            evs[0] = x.r[0].c[0];
            evs[1] = x.r[1].c[1];

            answerEvs = new double[(int) NumberOfRows(a)];
            answerEvs[0] = -1;
            answerEvs[1] = -2;

            SetsEqualEpsilon(evs, answerEvs, 0.00001, failures);
        }
    }

    public static void SetsEqualEpsilon(double[] a, double[] b, double epsilon, NumberReference failures) {
        double i, j;
        boolean found;

        for (i = 0d; i < a.length; i = i + 1d) {
            found = false;
            for (j = 0d; j < b.length; j = j + 1d) {
                if (mathEpsilonCompare(a[(int)i], b[(int)j], epsilon)) {
                    found = true;
                }
            }
            AssertTrue(found, failures);
        }
    }

    public static void testLUDecomposition(NumberReference failures){
        Matrix a, l, u, fl, fu, ui, li, i, fi;
        boolean success;

        a = CreateSquareMatrix(2d);
        l = CreateSquareMatrix(2d);
        u = CreateSquareMatrix(2d);

        a.r[0].c = nStringToNumberArray(" 4,  3".toCharArray());
        a.r[1].c = nStringToNumberArray(" 6,  3".toCharArray());

        success = LUDecomposition(a, l, u);
        AssertTrue(success, failures);

        fl = CreateSquareMatrix(2d);
        fl.r[0].c = nStringToNumberArray(" 1,  0".toCharArray());
        fl.r[1].c = nStringToNumberArray(" 1.5,  1".toCharArray());

        fu = CreateSquareMatrix(2d);
        fu.r[0].c = nStringToNumberArray(" 4,  3".toCharArray());
        fu.r[1].c = nStringToNumberArray(" 0,  -1.5".toCharArray());

        AssertTrue(MatrixEqualsEpsilon(l, fl,  0.001), failures);
        AssertTrue(MatrixEqualsEpsilon(l, fl,  0.001), failures);

        li = CreateSquareMatrix(2d);
        ui = CreateSquareMatrix(2d);

        InvertLowerTriangularMatrix(l, li);
        InvertUpperTriangularMatrix(u, ui);

        i = CreateSquareMatrix(2d);
        fi = CreateIdentityMatrix(2d);

        Multiply(i, l, li);
        AssertTrue(MatrixEqualsEpsilon(i, fi,  0.001), failures);
        Multiply(i, u, ui);
        AssertTrue(MatrixEqualsEpsilon(i, fi,  0.001), failures);
    }

    public static void TestSingularValueDecomposition1(NumberReference failures) {
        Matrix A;
        MatrixReference URef, SRef, VRef;
        boolean success;

        A = CreateSquareMatrix(3d);

        A.r[0].c = nStringToNumberArray("-1.000, 0.000, 0.000".toCharArray());
        A.r[1].c = nStringToNumberArray("-0.830, -0.557, 0.000".toCharArray());
        A.r[2].c = nStringToNumberArray("-0.691, -0.685, 0.232".toCharArray());

        URef = new MatrixReference();
        SRef = new MatrixReference();
        VRef = new MatrixReference();

        success = SingularValueDecomposition(A, URef, SRef, VRef);

        CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
    }

    public static void CheckSingularValueDecomposition(Matrix a, MatrixReference URef, MatrixReference SRef, MatrixReference VRef, boolean success, NumberReference failures) {
        Matrix U, S, V, Ac, Vt, I, Ut, Ic;
        double i;

        AssertTrue(success, failures);

        if(success){
            U = URef.matrix;
            S = SRef.matrix;
            V = VRef.matrix;

            Ac = MultiplyToNew(U, S);
            Vt = TransposeToNew(V);
            Ac = MultiplyToNew(Ac, Vt);

            AssertTrue(MatrixEqualsEpsilon(a, Ac, 0.0000001), failures);

            I = CreateSquareMatrix(NumberOfRows(U));
            for(i = 0d; i < min(NumberOfRows(U), NumberOfColumns(U)); i = i + 1d){
                I.r[(int)i].c[(int)i] = 1d;
            }
            Ut = TransposeToNew(U);
            Ic = MultiplyToNew(U, Ut);

            AssertTrue(MatrixEqualsEpsilon(I, Ic, 0.0000001), failures);

            I = CreateIdentityMatrix(NumberOfRows(V));
            Ic = MultiplyToNew(V, Vt);

            AssertTrue(MatrixEqualsEpsilon(I, Ic, 0.0000001), failures);
        }
    }

    public static void TestSingularValueDecomposition2(NumberReference failures) {
        Matrix A;
        MatrixReference URef, SRef, VRef;
        boolean success;

        A = CreateMatrix(2d, 3d);

        A.r[0].c = nStringToNumberArray("3, 2, 2".toCharArray());
        A.r[1].c = nStringToNumberArray("2, 3, -2".toCharArray());

        URef = new MatrixReference();
        SRef = new MatrixReference();
        VRef = new MatrixReference();

        success = SingularValueDecomposition(A, URef, SRef, VRef);

        CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
    }

    public static void TestSingularValueDecomposition3(NumberReference failures) {
        Matrix A;
        MatrixReference URef, SRef, VRef;
        boolean success;

        A = CreateSquareMatrix(3d);

        A.r[0].c = nStringToNumberArray("2, 0, 0".toCharArray());
        A.r[1].c = nStringToNumberArray("2, 1, 0".toCharArray());
        A.r[2].c = nStringToNumberArray("0, -2, 0".toCharArray());

        URef = new MatrixReference();
        SRef = new MatrixReference();
        VRef = new MatrixReference();

        success = SingularValueDecomposition(A, URef, SRef, VRef);

        CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
    }

    public static void TestSingularValueDecomposition4(NumberReference failures) {
        Matrix A;
        MatrixReference URef, SRef, VRef;
        boolean success;

        A = CreateMatrix(2d, 3d);

        A.r[0].c = nStringToNumberArray("1, 1, 0".toCharArray());
        A.r[1].c = nStringToNumberArray("1, 0, 1".toCharArray());

        URef = new MatrixReference();
        SRef = new MatrixReference();
        VRef = new MatrixReference();

        success = SingularValueDecomposition(A, URef, SRef, VRef);

        CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
    }

    public static void TestSingularValueDecomposition5(NumberReference failures) {
        Matrix A;
        MatrixReference URef, SRef, VRef;
        boolean success;

        A = CreateMatrix(3d, 2d);

        A.r[0].c = nStringToNumberArray("1, 1".toCharArray());
        A.r[1].c = nStringToNumberArray("1, 0".toCharArray());
        A.r[2].c = nStringToNumberArray("0, 1".toCharArray());

        URef = new MatrixReference();
        SRef = new MatrixReference();
        VRef = new MatrixReference();

        success = SingularValueDecomposition(A, URef, SRef, VRef);

        CheckSingularValueDecomposition(A, URef, SRef, VRef, success, failures);
    }
}































